package distribute

import (
	"fmt"
	omanager "github.com/odysseus/odysseus-protocol/gen/proto/go/nodemanager/v2"
	"sync"
	"testing"
	"time"
)

var (
	modelUrl = "http://18.167.103.232/admin/api/task/taskheat"
)

type demoWorkerManager struct {
	workers map[string]*omanager.NodeInfoResponse
	mux     sync.Mutex
}

func (m *demoWorkerManager) WorkerCount() int {
	m.mux.Lock()
	defer m.mux.Unlock()
	return len(m.workers)
}

func (m *demoWorkerManager) addWorker(addr string, info *omanager.NodeInfoResponse) {
	m.mux.Lock()
	defer m.mux.Unlock()
	m.workers[addr] = info
}

func (m *demoWorkerManager) ModelOperate(addr string, operate []*omanager.ModelOperate) error {
	for _, op := range operate {
		fmt.Printf("addr: %s - operate: %v\n", addr, op)
	}
	return nil
}

func generateWorkerManager() *demoWorkerManager {
	return &demoWorkerManager{
		workers: make(map[string]*omanager.NodeInfoResponse),
	}
}

func TestGetJson(t *testing.T) {
	infos, err := getModelInfo(modelUrl)
	if err != nil {
		t.Error(err)
	}
	for _, info := range infos {
		fmt.Printf("info.id = %d\n", info.TaskID)
	}
}

func generateAWorker() (string, *omanager.NodeInfoResponse) {
	addr := "0x4588Fa9951D44Ffd7Dd783885c83d34694fc5647"
	info := &omanager.NodeInfoResponse{
		Info: &omanager.NodeInfo{
			MinerPubkey:    "f41184d231af9eab0338a9deee5542c6efc0c18b99a0cfe6877e16fe453168c728164e4176dd1ff358c43ef2b87ed64e198876c8756dd35abeeddc375300c948",
			BenefitAddress: "0xeed5fd3046fa0a33f527e8ade3ae13e80629d436",
		},
		Models: &omanager.ModelsInfo{
			InstalledModels:     []*omanager.InstalledModel{},
			RunningModels:       []*omanager.RunningModel{},
			WaitToInstallModels: []*omanager.WaitToInstallModel{},
		},
		Hardware: &omanager.HardwareInfo{
			DISK: &omanager.DiskInfo{
				Total: 10 * 1024 * 1024 * 1024 * 1024,
				Free:  800,
				//Free:  8 * 1024 * 1024 * 1024 * 1024,
			},
			RAM: &omanager.MemoryInfo{
				Total: 128 * 1024 * 1024 * 1024,
				Free:  100 * 1024 * 1024 * 1024,
			},
			CPU: &omanager.CPUInfo{
				Model:   "Intel(R) Xeon(R) CPU E5-2686 v4 @ 2.30GHz",
				Number:  1,
				Cores:   8,
				Threads: 16,
				Usage:   10,
			},
			GPU: []*omanager.GPUInfo{
				&omanager.GPUInfo{
					Seq:         0,
					Uuid:        "111000",
					Model:       "Tesla V100-SXM2-16GB",
					Performance: 10000,
					PowerRating: 1000,
					PowerRt:     100,
					MemTotal:    16 * 1024 * 1024 * 1024,
					MemFree:     16 * 1024 * 1024 * 1024,
					Usage:       0,
					Temp:        20,
				},
			},
		},
	}
	return addr, info
}

func TestStartDistributor(t *testing.T) {
	manager := generateWorkerManager()
	infos, err := getModelInfo(modelUrl)
	if err != nil {
		t.Error(err)
		return
	}
	d := interStartDistributor(manager, infos)
	addr, info := generateAWorker()
	manager.addWorker(addr, info)
	d.AddWorker(0, addr, info)
	time.Sleep(time.Hour * 10)
}
